package controller

import (
	"electricity_bill_calc/model"
	"electricity_bill_calc/response"
	"electricity_bill_calc/security"
	"electricity_bill_calc/service"
	"fmt"
	"log"
	"net/http"
	"strconv"

	"github.com/gin-gonic/gin"
	"github.com/jinzhu/copier"
	"github.com/shopspring/decimal"
)

type _Meter04kVController struct {
	Router *gin.Engine
}

var Meter04kVController *_Meter04kVController

func InitializeMeter04kVController(router *gin.Engine) {
	Meter04kVController = &_Meter04kVController{
		Router: router,
	}

	Meter04kVController.Router.GET("/park/:pid/meter/template", security.EnterpriseAuthorize, download04kvMeterArchiveTemplate)
	Meter04kVController.Router.GET("/park/:pid/meters", security.EnterpriseAuthorize, ListPaged04kVMeter)
	Meter04kVController.Router.GET("/park/:pid/meter/:code", security.EnterpriseAuthorize, fetch04kVMeterDetail)
	Meter04kVController.Router.POST("/park/:pid/meter", security.EnterpriseAuthorize, createSingle04kVMeter)
	Meter04kVController.Router.PUT("/park/:pid/meter/:code", security.EnterpriseAuthorize, modifySingle04kVMeter)
}

func download04kvMeterArchiveTemplate(c *gin.Context) {
	result := response.NewResult(c)
	requestParkId := c.Param("pid")
	userSession, err := _retreiveSession(c)
	if err != nil {
		result.Unauthorized(err.Error())
		return
	}
	sure, err := service.ParkService.EnsurePark(userSession.Uid, requestParkId)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	if !sure {
		result.Unauthorized("不能访问不属于自己的园区。")
		return
	}
	parkDetail, err := service.ParkService.FetchParkDetail(requestParkId)
	if err != nil {
		result.NotFound("未找到指定的园区信息。")
		return
	}
	c.Status(http.StatusOK)
	c.Header("Content-Type", "application/octet-stream")
	c.Header("Content-Transfer-Encoding", "binary")
	c.FileAttachment("./assets/meter_04kv_template.xlsx", fmt.Sprintf("%s-户表档案.xlsx", parkDetail.Name))
}

func ListPaged04kVMeter(c *gin.Context) {
	result := response.NewResult(c)
	requestParkId := c.Param("pid")
	userSession, err := _retreiveSession(c)
	if err != nil {
		result.Unauthorized(err.Error())
		return
	}
	sure, err := service.ParkService.EnsurePark(userSession.Uid, requestParkId)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	if !sure {
		result.Unauthorized("不能访问不属于自己的园区。")
		return
	}
	requestPage, err := strconv.Atoi(c.DefaultQuery("page", "1"))
	if err != nil {
		result.NotAccept("查询参数[page]格式不正确。")
		return
	}
	requestKeyword := c.DefaultQuery("keyword", "")
	meters, totalItem, err := service.Meter04kVService.ListMeterDetail(requestParkId, requestKeyword, requestPage)
	if err != nil {
		result.NotFound(err.Error())
		return
	}
	result.Json(
		http.StatusOK,
		"已获取到符合条件的0.4kV表计集合。",
		response.NewPagedResponse(requestPage, totalItem).ToMap(),
		gin.H{"meters": meters},
	)
}

func fetch04kVMeterDetail(c *gin.Context) {
	result := response.NewResult(c)
	requestParkId := c.Param("pid")
	userSession, err := _retreiveSession(c)
	if err != nil {
		result.Unauthorized(err.Error())
		return
	}
	sure, err := service.ParkService.EnsurePark(userSession.Uid, requestParkId)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	if !sure {
		result.Unauthorized("不能访问不属于自己的园区。")
		return
	}
	requestMeterCode := c.Param("code")
	meter, err := service.Meter04kVService.Get04kVMeterDetail(requestParkId, requestMeterCode)
	if err != nil {
		result.NotFound(err.Error())
		return
	}
	if meter == nil {
		result.Json(http.StatusNotFound, "指定的表计信息未能找到。", gin.H{"meter": nil})
		return
	}
	result.Json(http.StatusOK, "指定的表计信息已找到。", gin.H{"meter": meter})
}

type _MeterModificationFormData struct {
	Address       *string         `json:"address" form:"address"`
	CustomerName  *string         `json:"customerName" form:"customerName"`
	ContactName   *string         `json:"contactName" form:"contactName"`
	ContactPhone  *string         `json:"contactPhone" form:"contactPhone"`
	Ratio         decimal.Decimal `json:"ratio" form:"ratio"`
	Seq           int             `json:"seq" form:"seq"`
	IsPublicMeter bool            `json:"isPublicMeter" form:"isPublicMeter"`
	WillDilute    bool            `json:"willDilute" form:"willDilute"`
	Enabled       bool            `json:"enabled" form:"enabled"`
}

type _MeterCreationFormData struct {
	Code          string          `json:"code" form:"code"`
	Address       *string         `json:"address" form:"address"`
	CustomerName  *string         `json:"customerName" form:"customerName"`
	ContactName   *string         `json:"contactName" form:"contactName"`
	ContactPhone  *string         `json:"contactPhone" form:"contactPhone"`
	Ratio         decimal.Decimal `json:"ratio" form:"ratio"`
	Seq           int             `json:"seq" form:"seq"`
	IsPublicMeter bool            `json:"isPublicMeter" form:"isPublicMeter"`
	WillDilute    bool            `json:"willDilute" form:"willDilute"`
	Enabled       bool            `json:"enabled" form:"enabled"`
}

func createSingle04kVMeter(c *gin.Context) {
	result := response.NewResult(c)
	requestParkId := c.Param("pid")
	userSession, err := _retreiveSession(c)
	if err != nil {
		result.Unauthorized(err.Error())
		return
	}
	sure, err := service.ParkService.EnsurePark(userSession.Uid, requestParkId)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	if !sure {
		result.Unauthorized("不能访问不属于自己的园区。")
		return
	}
	formData := new(_MeterCreationFormData)
	c.BindJSON(formData)
	log.Printf("[controller|debug] form: %+v", formData)
	newMeter := new(model.Meter04KV)
	copier.Copy(newMeter, formData)
	newMeter.ParkId = requestParkId
	log.Printf("[controller|debug] meter: %+v", newMeter)
	err = service.Meter04kVService.CreateSingleMeter(*newMeter)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	result.Created("新0.4kV表计已经添加完成。")
}

func modifySingle04kVMeter(c *gin.Context) {
	result := response.NewResult(c)
	requestParkId := c.Param("pid")
	userSession, err := _retreiveSession(c)
	if err != nil {
		result.Unauthorized(err.Error())
		return
	}
	sure, err := service.ParkService.EnsurePark(userSession.Uid, requestParkId)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	if !sure {
		result.Unauthorized("不能访问不属于自己的园区。")
		return
	}
	requestMeterCode := c.Param("code")
	meterDetail, err := service.Meter04kVService.Get04kVMeterDetail(requestParkId, requestMeterCode)
	if err != nil {
		result.NotFound(err.Error())
		return
	}
	if meterDetail == nil {
		result.NotFound("指定表计的信息为找到，不能修改。")
		return
	}
	formData := new(_MeterModificationFormData)
	c.BindJSON(formData)
	copier.Copy(meterDetail, formData)
	err = service.Meter04kVService.UpdateSingleMeter(meterDetail)
	if err != nil {
		result.Error(http.StatusInternalServerError, err.Error())
		return
	}
	result.Updated("指定0.4kV表计信息已经更新。")
}
